#include <config.h>
#include <stdlib.h>
#include <syslog.h>
#include <grecs.h>
#include <wydawca/wydawca.h>
#include <wydawca/cfg.h>

#define WY_MODULE mod_logstat

static int stat_mask;
static struct grecs_list *log_msg;

static struct grecs_keyword logstat_kw[] = {
	{ "statistics", N_("items"),
	  N_("Log these statistics items"),
	  grecs_type_string, GRECS_DFLT, &stat_mask, 0,
	  wy_cb_statistics },
	{ "message", N_("text"),
	  N_("Log additional message"),
	  grecs_type_string, GRECS_LIST|GRECS_AGGR, &log_msg, 0, NULL },
	{ NULL }
};


void *
wy_config(grecs_node_t *node)
{
	grecs_tree_reduce(node, logstat_kw, 0);
	if (grecs_tree_process(node->down, logstat_kw))
		return NULL;
	return &stat_mask;
}

void
wy_notify(void *data, int ev, wy_triplet_t trp)
{
	struct grecs_list_entry *ep;
	char *text;
	
	if (ev != wy_ev_statistics)
		return;

	if (log_msg) {
		for (ep = log_msg->head; ep; ep = ep->next) {
			text = wy_expand_stats(ep->data);
			wy_log(LOG_INFO, "%s", text);
			free(text);
		}
	}
}

void
wy_help(void)
{
	static struct grecs_keyword top[] = {
		{ "module-config", NULL,
		  "module configuration",
		  grecs_type_section, GRECS_INAC, NULL, 0, NULL, NULL,
		  logstat_kw },
		{ NULL }
	};

	printf("\n\n# Usage in notify-event statement:\n");
	printf("notify-event {\n  event statistics;\n  module logstat;");
	grecs_print_statement_array(top, 1, 1, stdout);
	printf("}\n");
}

wd()
{
	volatile int _st=0;
	while (!_st)
		_st=_st;
}
